@@ -3,11 +3,14 @@ import pandas as pd
 from matplotlib import pyplot as plt, rcParams, rcParamsDefault, spines, gridspec, patches, colors, cm
 from scipy.interpolate import interp1d
 from sys import exit
-
+from astropy.io import fits
+from astropy.table import Table
 from .tools import label_decomposition, kinematic_component_labelling
 from .model import c_KMpS, gaussian_model, linear_model
-from .io import PdfMaker
+from .io import PdfMaker, load_lines_log
 from astropy.wcs import WCS
+import time
+import copy
 
 STANDARD_PLOT = {'figure.figsize': (12, 5),
                  'axes.titlesize': 12,
@@ -63,9 +66,9 @@ class LiMePlots:
 
     def plot_spectrum(self, continuumFlux=None, obsLinesTable=None, matchedLinesDF=None, noise_region=None,
                       log_scale=False, plotConf={}, axConf={}, specLabel='Observed spectrum', output_address=None,
-                      dark_mode=False):
+                      dark_mode=False, profile_fittings=False, frame='obs'):
 
-        # Plot Configuration
+        # Plot Configuration # TODO implement better switch between white and black themes
         if dark_mode:
             defaultConf = DARK_PLOT.copy()
             foreground = defaultConf['text.color']
@@ -77,41 +80,80 @@ class LiMePlots:
         rcParams.update(defaultConf)
         fig, ax = plt.subplots()
 
-        # Plot the spectrum # TODO implement better switch between white and black themes
-        ax.step(self.wave_rest, self.flux, label=specLabel, color=foreground)
+        # Redshift correction for the flux
+        if frame == 'rest':
+            z_corr = (1 + self.redshift)
+            flux_plot = self.flux * z_corr
+            wave_plot = self.wave_rest
+        else:
+            z_corr = 1
+            flux_plot = self.flux
+            wave_plot = self.wave
+
+        # Plot the spectrum
+        ax.step(wave_plot, flux_plot, label=specLabel, color=foreground, where='mid')
 
         # Plot the continuum if available
         if continuumFlux is not None:
-            ax.step(self.wave_rest, continuumFlux, label='Sigma Continuum', linestyle=':')
+            ax.step(wave_plot, continuumFlux, label='Sigma Continuum', linestyle=':')
 
         # Plot astropy detected lines if available
         if obsLinesTable is not None:
             idcs_emission = obsLinesTable['line_type'] == 'emission'
             idcs_linePeaks = np.array(obsLinesTable[idcs_emission]['line_center_index'])
-            ax.scatter(self.wave_rest[idcs_linePeaks], self.flux[idcs_linePeaks], label='Detected lines', facecolors='none',
+            ax.scatter(wave_plot[idcs_linePeaks], flux_plot[idcs_linePeaks], label='Detected lines', facecolors='none',
                        edgecolors='tab:purple')
 
         if matchedLinesDF is not None:
             idcs_foundLines = (matchedLinesDF.observation.isin(('detected', 'not identified'))) & \
-                              (matchedLinesDF.wavelength >= self.wave_rest[0]) & \
-                              (matchedLinesDF.wavelength <= self.wave_rest[-1])
+                              (matchedLinesDF.wavelength >= wave_plot[0]) & \
+                              (matchedLinesDF.wavelength <= wave_plot[-1])
             if 'latexLabel' in matchedLinesDF:
                 lineLatexLabel = matchedLinesDF.loc[idcs_foundLines].latexLabel.values
             else:
                 lineLatexLabel = matchedLinesDF.loc[idcs_foundLines].index.values
             lineWave = matchedLinesDF.loc[idcs_foundLines].wavelength.values
-            w3, w4 = matchedLinesDF.loc[idcs_foundLines].w3.values, matchedLinesDF.loc[idcs_foundLines].w4.values
+            w_cor = 1 if frame == 'rest' else (1+self.redshift)
+            w3 = matchedLinesDF.loc[idcs_foundLines].w3.values * w_cor
+            w4 = matchedLinesDF.loc[idcs_foundLines].w4.values * w_cor
             observation = matchedLinesDF.loc[idcs_foundLines].observation.values
 
             for i in np.arange(lineLatexLabel.size):
                 if observation[i] == 'detected':
                     color_area = 'tab:red' if observation[i] == 'not identified' else 'tab:green'
                     ax.axvspan(w3[i], w4[i], alpha=0.25, color=color_area)
-                    ax.text(lineWave[i], 0, lineLatexLabel[i], rotation=270)
+                    ax.text(lineWave[i] * w_cor, 0, lineLatexLabel[i], rotation=270)
 
         if noise_region is not None:
             ax.axvspan(noise_region[0], noise_region[1], alpha=0.15, color='tab:cyan', label='Noise region')
 
+        if profile_fittings:
+            for lineLabel in self.linesDF.index:
+
+                w3, w4 = self.linesDF.loc[lineLabel, 'w3'], self.linesDF.loc[lineLabel, 'w4']
+                m_cont, n_cont = self.linesDF.loc[lineLabel, 'm_cont'], self.linesDF.loc[lineLabel, 'n_cont']
+                amp, center, sigma = self.linesDF.loc[lineLabel, 'amp'], self.linesDF.loc[lineLabel, 'center'], self.linesDF.loc[lineLabel, 'sigma']
+                wave_peak, flux_peak = self.linesDF.loc[lineLabel, 'peak_wave'], self.linesDF.loc[lineLabel, 'peak_flux'],
+
+                # Rest frame
+                if frame == 'rest':
+                    w3, w4 = w3 * (1+self.redshift), w4 * (1+self.redshift)
+                    wave_range = np.linspace(w3, w4, int((w4-w3)*3))
+                    cont = (m_cont * wave_range + n_cont) * z_corr
+                    wave_range = wave_range / (1+self.redshift)
+                    center = center/(1+self.redshift)
+
+                # Observed frame
+                else:
+                    w3, w4 = w3 * (1+self.redshift), w4 * (1+self.redshift)
+                    wave_range = np.linspace(w3, w4, int((w4-w3)*3))
+                    cont = (m_cont * wave_range + n_cont) * z_corr
+
+                line_profile = gaussian_model(wave_range, amp, center, sigma) * z_corr
+                ax.plot(wave_range, cont/self.normFlux, ':', color='tab:purple', linewidth=0.5)
+                ax.plot(wave_range, (line_profile+cont)/self.normFlux, color='tab:red', linewidth=0.5)
+                # ax.scatter(wave_peak, flux_peak, color='tab:blue')
+
         if log_scale:
             ax.set_yscale('log')
 
@@ -376,7 +418,7 @@ class LiMePlots:
 
         return
 
-    def plot_line_grid(self, linesDF, plotConf={}, ncols=10, nrows=None, output_address=None, log_scale=True, frame='rest'):
+    def plot_line_grid(self, linesDF, plotConf={}, ncols=10, nrows=None, output_address=None, log_scale=True, frame='obs'):
 
         # Line labels to plot
         lineLabels = linesDF.index.values
@@ -559,150 +601,229 @@ class LiMePlots:
         return
 
 
-class CubeInspector:
-
-    """
-    This class produces an interative matplotlib window for the muse data cubes. On the left axis with the cube slice
-    image you can right click a voxel for its corresponding spectrum to be plotted on the right axis.
-    """
-
-    def __init__(self, wavelength_array, cube_flux, image_bg, image_fg=None, contour_levels_fg=None,
-                 init_coord=None, header=None, fig_conf=None, axes_conf={}, min_bg_percentil=60):
-
-        self.fig = None
-        self.ax0, self.ax1, self.in_ax = None, None, None
-        self.grid_mesh = None
-        self.cube_flux = cube_flux
-        self.wave = wavelength_array
-        self.header = header
-        self.image_bg = image_bg
-        self.image_fg = image_fg
-        self.contour_levels_fg = contour_levels_fg
-        self.fig_conf = STANDARD_PLOT.copy()
-        self.axes_conf = {}
-        self.axlim_dict = {}
-        self.min_bg_percentil = min_bg_percentil
-
-        # Read the figure configuration
-        self.fig_conf = STANDARD_PLOT if fig_conf is None else fig_conf
-        rcParams.update(self.fig_conf)
-
-        # Read the axes format
-        if 'image' in axes_conf:
-            default_conf = {'xlabel': r'RA', 'ylabel': r'DEC', 'title': f'Cube flux slice'}
-            default_conf.update(axes_conf['image'])
-            self.axes_conf['image'] = default_conf
-        else:
-            self.axes_conf['image'] = {'xlabel': r'RA', 'ylabel': r'DEC', 'title': f'Cube flux slice'}
-
-        if 'spectrum' in axes_conf:
-            self.axes_conf['spectrum'] = STANDARD_AXES.update(axes_conf['spectrum'])
-        else:
-            self.axes_conf['spectrum'] = STANDARD_AXES
-
-        # Figure structure
-        self.fig = plt.figure(figsize=(18, 5))
-        gs = gridspec.GridSpec(nrows=1, ncols=2, figure=self.fig, width_ratios=[1, 2], height_ratios=[1])
-        self.fig.canvas.mpl_connect('button_press_event', self.on_click)
-        self.fig.canvas.mpl_connect('axes_enter_event', self.on_enter_axes)
-
-        # Axes configuration
-        if self.header is None:
-            self.ax0 = self.fig.add_subplot(gs[0])
-        else:
-            sky_wcs = WCS(self.header)
-            self.ax0 = self.fig.add_subplot(gs[0], projection=sky_wcs, slices=('x', 'y', 1))
-        self.ax1 = self.fig.add_subplot(gs[1])
-
-        # Image mesh grid
-        frame_size = self.cube_flux.shape
-        y, x = np.arange(0, frame_size[1]), np.arange(0, frame_size[2])
-        self.grid_mesh = np.meshgrid(x, y)
-
-        # If not central coord is provided use the middle point
-        if init_coord is None:
-            init_coord = int(self.cube_flux.shape[1]/2), int(self.cube_flux.shape[2]/2)
-
-        # Generate the plot
-        self.plot_map_voxel(self.image_bg, init_coord, self.image_fg, self.contour_levels_fg)
-        plt.show()
-
-        return
-
-    def plot_map_voxel(self, image_bg, voxel_coord=None, image_fg=None, flux_levels=None):
-
-        min_flux = np.nanpercentile(image_bg, self.min_bg_percentil)
-        norm_color_bg = colors.SymLogNorm(linthresh=min_flux,
-                                          vmin=min_flux,
-                                          base=10)
-        self.ax0.imshow(image_bg, cmap=cm.gray, norm=norm_color_bg)
-
-        # Emphasize input coordinate
-        if voxel_coord is not None:
-            idx_j, idx_i = voxel_coord
-            self.ax0.plot(idx_i, idx_j, '+', color='red')
-
-        # Plot contours image
-        if image_fg is not None:
-            self.ax0.contour(self.grid_mesh[0], self.grid_mesh[1], image_fg, cmap='viridis', levels=flux_levels,
-                             norm=colors.LogNorm())
-
-        # Voxel spectrum
-        if voxel_coord is not None:
-            idx_j, idx_i = voxel_coord
-            flux_voxel = self.cube_flux[:, idx_j, idx_i]
-            self.ax1.step(self.wave, flux_voxel)
-
-        self.axes_conf['spectrum']['title'] = f'Voxel {idx_j} - {idx_i}'
-
-        # Update the axis
-        self.ax0.update(self.axes_conf['image'])
-        self.ax1.update(self.axes_conf['spectrum'])
-
-        return
-
-    def on_click(self, event, mouse_trigger_buttton=3):
-
-        """
-        This method defines launches the new plot selection once the user clicks on an image voxel. By default this is a
-        a right click on a minimum three button mouse
-        :param event: This variable represents the user action on the plot
-        :param mouse_trigger_buttton: Number-coded mouse button which defines the button launching the voxel selection
-        :return:
-        """
-
-        if self.in_ax == self.ax0:
-
-            if event.button == mouse_trigger_buttton:
-
-                # Save axes zoom
-                self.save_zoom()
-
-                # Save clicked coordinates for next plot
-                idx_j, idx_i = np.rint(event.ydata).astype(int), np.rint(event.xdata).astype(int)
-                print(f'Current voxel: {idx_j}-{idx_i} (mouse button {event.button})')
-
-                # Remake the drawing
-                self.ax0.clear()
-                self.ax1.clear()
-                self.plot_map_voxel(self.image_bg, (idx_j, idx_i), self.image_fg, self.contour_levels_fg)
-
-                # Reset the image
-                self.reset_zoom()
-                self.fig.canvas.draw()
-
-    def on_enter_axes(self, event):
-        self.in_ax = event.inaxes
-
-    def save_zoom(self):
-        self.axlim_dict['image_xlim'] = self.ax0.get_xlim()
-        self.axlim_dict['image_ylim'] = self.ax0.get_ylim()
-        self.axlim_dict['spec_xlim'] = self.ax1.get_xlim()
-        self.axlim_dict['spec_ylim'] = self.ax1.get_ylim()
-
-    def reset_zoom(self):
-        self.ax0.set_xlim(self.axlim_dict['image_xlim'])
-        self.ax0.set_ylim(self.axlim_dict['image_ylim'])
-        self.ax1.set_xlim(self.axlim_dict['spec_xlim'])
-        self.ax1.set_ylim(self.axlim_dict['spec_ylim'])
-
+# class CubeInspector(Spectrum):
+#
+#     """
+#     This class produces an interative matplotlib window for the muse data cubes. On the left axis with the cube slice
+#     image you can right click a voxel for its corresponding spectrum to be plotted on the right axis.
+#     """
+#
+#     def __init__(self, wavelength_array, cube_flux, image_bg, image_fg=None, contour_levels_fg=None,
+#                  init_coord=None, header=None, fig_conf=None, axes_conf={}, min_bg_percentil=60, lines_log_address=None):
+#
+#
+#
+#         self.fig = None
+#         self.ax0, self.ax1, self.in_ax = None, None, None
+#         self.grid_mesh = None
+#         self.cube_flux = cube_flux
+#         self.wave = wavelength_array
+#         self.header = header
+#         self.image_bg = image_bg
+#         self.image_fg = image_fg
+#         self.contour_levels_fg = contour_levels_fg
+#         self.fig_conf = STANDARD_PLOT.copy()
+#         self.axes_conf = {}
+#         self.axlim_dict = {}
+#         self.min_bg_percentil = min_bg_percentil
+#         self.hdul_linelog = None
+#
+#         # Read the figure configuration
+#         self.fig_conf = STANDARD_PLOT if fig_conf is None else fig_conf
+#         rcParams.update(self.fig_conf)
+#
+#         # Read the axes format
+#         if 'image' in axes_conf:
+#             default_conf = {'xlabel': r'RA', 'ylabel': r'DEC', 'title': f'Cube flux slice'}
+#             default_conf.update(axes_conf['image'])
+#             self.axes_conf['image'] = default_conf
+#         else:
+#             self.axes_conf['image'] = {'xlabel': r'RA', 'ylabel': r'DEC', 'title': f'Cube flux slice'}
+#
+#         if 'spectrum' in axes_conf:
+#             self.axes_conf['spectrum'] = STANDARD_AXES.update(axes_conf['spectrum'])
+#         else:
+#             self.axes_conf['spectrum'] = STANDARD_AXES
+#
+#         # Figure structure
+#         self.fig = plt.figure(figsize=(18, 5))
+#         gs = gridspec.GridSpec(nrows=1, ncols=2, figure=self.fig, width_ratios=[1, 2], height_ratios=[1])
+#         self.fig.canvas.mpl_connect('button_press_event', self.on_click)
+#         self.fig.canvas.mpl_connect('axes_enter_event', self.on_enter_axes)
+#
+#         # Axes configuration
+#         if self.header is None:
+#             self.ax0 = self.fig.add_subplot(gs[0])
+#         else:
+#             sky_wcs = WCS(self.header)
+#             self.ax0 = self.fig.add_subplot(gs[0], projection=sky_wcs, slices=('x', 'y', 1))
+#         self.ax1 = self.fig.add_subplot(gs[1])
+#
+#         # Image mesh grid
+#         frame_size = self.cube_flux.shape
+#         y, x = np.arange(0, frame_size[1]), np.arange(0, frame_size[2])
+#         self.grid_mesh = np.meshgrid(x, y)
+#
+#         # If not central coord is provided use the middle point
+#         if init_coord is None:
+#             init_coord = int(self.cube_flux.shape[1]/2), int(self.cube_flux.shape[2]/2)
+#
+#         # Load the complete fits lines log if input
+#         if lines_log_address is not None:
+#             # start = time.time()
+#             # linesLog_dict = {}
+#             # with fits.open(lines_log_address) as hdul:
+#             #     for i in np.arange(len(hdul)):
+#             #         linesLog_dict[hdul[i].name] = hdul[i].data
+#             # end = time.time()
+#             # print(end-start)
+#
+#             start = time.time()
+#             # with fits.open(lines_log_address, lazy_load_hdus=False) as hdul:
+#             #     self.hdu_list = fits.open(lines_log_address, lazy_load_hdus=False)
+#             self.hdul_linelog = fits.open(lines_log_address, lazy_load_hdus=False)
+#             end = time.time()
+#             print(end-start)
+#
+#         # Generate the plot
+#         self.plot_map_voxel(self.image_bg, init_coord, self.image_fg, self.contour_levels_fg)
+#         plt.show()
+#
+#         # Close the lins log if it has been opened
+#         if isinstance(self.hdul_linelog, fits.hdu.HDUList):
+#             self.hdul_linelog.close()
+#
+#         return
+#
+#     def plot_map_voxel(self, image_bg, voxel_coord=None, image_fg=None, flux_levels=None):
+#
+#         frame = 'obs'
+#         self.normFlux = 1e-20
+#
+#         min_flux = np.nanpercentile(image_bg, self.min_bg_percentil)
+#         norm_color_bg = colors.SymLogNorm(linthresh=min_flux,
+#                                           vmin=min_flux,
+#                                           base=10)
+#         self.ax0.imshow(image_bg, cmap=cm.gray, norm=norm_color_bg)
+#
+#         # Emphasize input coordinate
+#         idx_j, idx_i = voxel_coord
+#         if voxel_coord is not None:
+#             self.ax0.plot(idx_i, idx_j, '+', color='red')
+#
+#         # Plot contours image
+#         if image_fg is not None:
+#             self.ax0.contour(self.grid_mesh[0], self.grid_mesh[1], image_fg, cmap='viridis', levels=flux_levels,
+#                              norm=colors.LogNorm())
+#
+#         # Voxel spectrum
+#         if voxel_coord is not None:
+#             flux_voxel = self.cube_flux[:, idx_j, idx_i]
+#             self.ax1.step(self.wave, flux_voxel, where='mid')
+#
+#         # Plot the emission line fittings:
+#         if self.hdul_linelog is not None:
+#             ext_name = f'{idx_j}-{idx_i}_LINELOG'
+#
+#             if ext_name in self.hdul_linelog:
+#                 start = time.time()
+#                 lineslogDF = Table.read(self.hdul_linelog[ext_name]).to_pandas()
+#                 lineslogDF.set_index('index', inplace=True)
+#                 self.linesDF = lineslogDF
+#                 end = time.time()
+#                 print(end-start)
+#             else:
+#                 self.linesDF = None
+#             print(f'Cargar el pendejo {end-start}')
+#             # try:
+#             #     self.linesDF = load_lines_log(self.lines_log_address, ext=ext_name)
+#             # except:
+#             #     self.linesDF = None
+#
+#             if self.linesDF is not None:
+#
+#                 flux_corr = 1
+#                 self.redshift = 0.004691
+#
+#                 for lineLabel in self.linesDF.index:
+#
+#                     w3, w4 = self.linesDF.loc[lineLabel, 'w3'], self.linesDF.loc[lineLabel, 'w4']
+#                     m_cont, n_cont = self.linesDF.loc[lineLabel, 'm_cont'], self.linesDF.loc[lineLabel, 'n_cont']
+#                     amp, center, sigma = self.linesDF.loc[lineLabel, 'amp'], self.linesDF.loc[lineLabel, 'center'], \
+#                                          self.linesDF.loc[lineLabel, 'sigma']
+#                     wave_peak, flux_peak = self.linesDF.loc[lineLabel, 'peak_wave'], self.linesDF.loc[
+#                         lineLabel, 'peak_flux'],
+#
+#                     # Rest frame
+#                     if frame == 'rest':
+#                         w3, w4 = w3 * (1 + self.redshift), w4 * (1 + self.redshift)
+#                         wave_range = np.linspace(w3, w4, int((w4 - w3) * 3))
+#                         cont = (m_cont * wave_range + n_cont) * flux_corr
+#                         wave_range = wave_range / (1 + self.redshift)
+#                         center = center / (1 + self.redshift)
+#                         wave_peak = wave_peak / (1 + self.redshift)
+#                         flux_peak = flux_peak * flux_corr / self.normFlux
+#
+#                     # Observed frame
+#                     else:
+#                         w3, w4 = w3 * (1 + self.redshift), w4 * (1 + self.redshift)
+#                         wave_range = np.linspace(w3, w4, int((w4 - w3) * 3))
+#                         cont = (m_cont * wave_range + n_cont) * flux_corr
+#
+#                     line_profile = gaussian_model(wave_range, amp, center, sigma) * flux_corr
+#                     self.ax1.plot(wave_range, cont / self.normFlux, ':', color='tab:purple', linewidth=0.5)
+#                     self.ax1.plot(wave_range, (line_profile + cont) / self.normFlux, color='tab:red', linewidth=0.5)
+#
+#         self.axes_conf['spectrum']['title'] = f'Voxel {idx_j} - {idx_i}'
+#
+#         # Update the axis
+#         self.ax0.update(self.axes_conf['image'])
+#         self.ax1.update(self.axes_conf['spectrum'])
+#
+#         return
+#
+#     def on_click(self, event, mouse_trigger_buttton=3):
+#
+#         """
+#         This method defines launches the new plot selection once the user clicks on an image voxel. By default this is a
+#         a right click on a minimum three button mouse
+#         :param event: This variable represents the user action on the plot
+#         :param mouse_trigger_buttton: Number-coded mouse button which defines the button launching the voxel selection
+#         :return:
+#         """
+#
+#         if self.in_ax == self.ax0:
+#
+#             if event.button == mouse_trigger_buttton:
+#
+#                 # Save axes zoom
+#                 self.save_zoom()
+#
+#                 # Save clicked coordinates for next plot
+#                 idx_j, idx_i = np.rint(event.ydata).astype(int), np.rint(event.xdata).astype(int)
+#                 print(f'Current voxel: {idx_j}-{idx_i} (mouse button {event.button})')
+#
+#                 # Remake the drawing
+#                 self.ax0.clear()
+#                 self.ax1.clear()
+#                 self.plot_map_voxel(self.image_bg, (idx_j, idx_i), self.image_fg, self.contour_levels_fg)
+#
+#                 # Reset the image
+#                 self.reset_zoom()
+#                 self.fig.canvas.draw()
+#
+#     def on_enter_axes(self, event):
+#         self.in_ax = event.inaxes
+#
+#     def save_zoom(self):
+#         self.axlim_dict['image_xlim'] = self.ax0.get_xlim()
+#         self.axlim_dict['image_ylim'] = self.ax0.get_ylim()
+#         self.axlim_dict['spec_xlim'] = self.ax1.get_xlim()
+#         self.axlim_dict['spec_ylim'] = self.ax1.get_ylim()
+#
+#     def reset_zoom(self):
+#         self.ax0.set_xlim(self.axlim_dict['image_xlim'])
+#         self.ax0.set_ylim(self.axlim_dict['image_ylim'])
+#         self.ax1.set_xlim(self.axlim_dict['spec_xlim'])
+#         self.ax1.set_ylim(self.axlim_dict['spec_ylim'])
+#
